#include "serverpool.h"
#include "state.pb.h"

using namespace std;

ServerPool::ServerPool(int size)
	: servers(new Server[size]),
	  num_servers(size),
	  clients_added(0)
{
}

ServerPool::~ServerPool()
{
	delete[] servers;
}
	
CubemapStateProto ServerPool::serialize()
{
	CubemapStateProto state;

	for (int i = 0; i < num_servers; ++i) {
                CubemapStateProto local_state = servers[i].serialize();

		// The stream state should be identical between the servers, so we only store it once.
		if (i == 0) {
			state.mutable_streams()->MergeFrom(local_state.streams());
		}
		for (int j = 0; j < local_state.clients_size(); ++j) {
			state.add_clients()->MergeFrom(local_state.clients(j));
		}
        }

	return state;
}

void ServerPool::add_client(int sock)
{
	servers[clients_added++ % num_servers].add_client_deferred(sock);
}

void ServerPool::add_client_from_serialized(const ClientProto &client)
{
	servers[clients_added++ % num_servers].add_client_from_serialized(client);
}

void ServerPool::add_stream(const std::string &stream_id)
{
	for (int i = 0; i < num_servers; ++i) {
		servers[i].add_stream(stream_id);
	}
}

void ServerPool::add_stream_from_serialized(const StreamProto &stream)
{
	for (int i = 0; i < num_servers; ++i) {
		servers[i].add_stream_from_serialized(stream);
	}
}

void ServerPool::set_header(const std::string &stream_id, const std::string &header)
{
	for (int i = 0; i < num_servers; ++i) {
		servers[i].set_header(stream_id, header);
	}
}

void ServerPool::add_data(const std::string &stream_id, const char *data, size_t bytes)
{
	for (int i = 0; i < num_servers; ++i) {
		servers[i].add_data_deferred(stream_id, data, bytes);
	}
}

void ServerPool::run()
{
	for (int i = 0; i < num_servers; ++i) {
		servers[i].run();
	}
}
	
void ServerPool::stop()
{
	for (int i = 0; i < num_servers; ++i) {
		servers[i].stop();
	}
}
	
vector<ClientStats> ServerPool::get_client_stats() const
{
	vector<ClientStats> ret;
	for (int i = 0; i < num_servers; ++i) {
		vector<ClientStats> stats = servers[i].get_client_stats();
		ret.insert(ret.end(), stats.begin(), stats.end());
	}
	return ret;
}
	
void ServerPool::set_mark_pool(const std::string &stream_id, MarkPool *mark_pool)
{
	for (int i = 0; i < num_servers; ++i) {
		servers[i].set_mark_pool(stream_id, mark_pool);
	}	
}
